#include <ouroboros/device_tree.tpp>
#include <ouroboros/function_manager.h>
#include <ouroboros/data/base.hpp>
#include <ouroboros/data/base_string.h>
#include <ouroboros/data/base_enum.h>
#include <ouroboros/data/base_boolean.h>
#include <ouroboros/data/base_number.hpp>
#include <ouroboros/data/base_floating.hpp>
#include <ouroboros/data/base_function.h>
#include <stdint.h>
#include <string>
#include <cmath>

namespace ouroboros
{
   namespace detail
   {
      template<>
      void free_tree(group<var_field>& aRoot)
      {
         std::vector<var_field*> currentFields = aRoot.getFields();
         std::vector<group<var_field>*> currentGroups =
         aRoot.getGroups();
         for (std::size_t i = 0; i < currentFields.size(); ++i)
         {
            delete currentFields[i];
         }
         for (std::size_t i = 0; i < currentGroups.size(); ++i)
         {
            free_tree(*currentGroups[i]);
            delete currentGroups[i];
         }
      }
      template<>
      group<var_field> build_tree(function_manager& aManager)
      {
         std::vector<std::string> params;
         group<var_field> result("server", "Root node.");
         std::map<std::string, int> enumMap;
         group<var_field> *home = new group<var_field>("Home", "This is the Home group.");
         home->add(new base_number<int32_t, -2147483648, 2147483647 >(
            "A number",
            "A number for use in the program.",
            20,
            std::pair<int32_t,int32_t>(-2147483648, 2147483647)));
         home->add(new base_number<uint16_t, 0, 65535 >(
            "A number2",
            "A number for use in the program.",
            20,
            std::pair<uint16_t,uint16_t>(20, 65535)));
         home->add(new base_string(
            "A string",
            "A string for use in the program.",
            "hello",
            ".*",
            std::pair<std::size_t, std::size_t>(std::numeric_limits<std::size_t>::min(), std::numeric_limits<std::size_t>::max())));
         enumMap.clear();
         enumMap["Enum1"] = 0;
         enumMap["Enum2"] = 1;
         enumMap["Enum3"] = 2;
         enumMap["Enum4"] = 3;
         enumMap["Enum5"] = 4;
         home->add(new base_enum(
            "An enum",
            "An enum for use in the program.",
            0,
            enumMap));
         home->add(new base_boolean(
            "A boolean",
            "An boolean for use in the program.",
            true));
         result.add(home);
         group<var_field> *general = new group<var_field>("General", "This is General, a top level nav item.");
         group<var_field> *advanced = new group<var_field>("Advanced", "Advanced is nested inside General.");
         advanced->add(new base_string(
            "A string inside Advanced",
            "A string for use in the program.",
            "hello",
            ".*",
            std::pair<std::size_t, std::size_t>(std::numeric_limits<std::size_t>::min(), std::numeric_limits<std::size_t>::max())));
         advanced->add(new base_floating<float>(
            "A float",
            "A floating number",
            12345.6789,
            std::pair<float,float>(0, 1000000000)));
         advanced->add(new base_floating<double>(
            "A double",
            "A double precision number",
            9876.5432,
            std::pair<double,double>(-1000000000, 1000000000)));
         group<var_field> *veryadvanced = new group<var_field>("Very Advanced", "This is doubly-nested inside General.");
         veryadvanced->add(new base_string(
            "Double Nested",
            "Bottom of the tree.",
            "hello",
            ".*",
            std::pair<std::size_t, std::size_t>(std::numeric_limits<std::size_t>::min(), std::numeric_limits<std::size_t>::max())));
         advanced->add(veryadvanced);
         general->add(advanced);
         general->add(new base_string(
            "A string inside group 1",
            "A string for use in the program.",
            "hello",
            ".*",
            std::pair<std::size_t, std::size_t>(std::numeric_limits<std::size_t>::min(), std::numeric_limits<std::size_t>::max())));
         result.add(general);

         return result;
      }
   }
}
